/*Copyright  2015, SAS Institute Inc., Cary, NC, USA.  All Rights Reserved. */


/* ****************************************************** */
/* ugip_acm_datamart.sas                                  */
/*                                                        */
/* This job updates various tables within the SAS Envi-   */
/* ronment Manager Data Mart to bring them in line with   */
/* the new data mart schema.  Most of these changes are   */
/* minor (e.g. assigning formats, adjusting lengths, etc.)*/
/*                                                        */
/* All of the libraries within the data mart are updated, */
/* and a report listing is generated if unexpected tables */
/* are encountered or if an expected table was not found. */
/*                                                        */
/* Report output is NOT generted for tables that do not   */
/* require any changes.  A small number of obsolete tables*/
/* may remain in the data mart after this job runs.  This */
/* is intended to prevent upgrade-in-place job from poten-*/
/* tialing "breaking" any site-specific jobs that may use */
/* one of these tables.                                   */
/*                                                        */
/* ****************************************************** */


/* *********************** Internal Macros (used only here) **********************/

%macro fixartds(data=,dropvars=);

 /* This macro adds the ARTIFACT data extacted by   */
 /* the earlier version to the new ARTIFACT tables. */
 /* This macro also ensures the proper formats are  */
 /* assigned to some variables.                     */

  %if %sysfunc(exist(artifact.&data)) %then
  %do; /* table exists - fix it */

     data artifact.&data(compress=yes);
      format _loadt datetime datetimeMinute startdt stopdt datetime. shift shifttxt.;
      set artifact.&data ;
      %if "&dropvars" ne "" %then
      %do; /* generate DROP statement */
         drop &dropvars;
      %end;/* generate DROP statement */
     run;

  %end;/* table exists - fix it */
  %else
  %do; /* table does NOT exist */

     %put %str(WARNING: The specified table (&data) was NOT found; therefore, its structure could NOT be revised to the current schema.);

     %tableMissing(lib=ARTIFACT,table=&data);
          

  %end;/* table does NOT exist */

%mend fixartds;

%macro fixrptds(data=);

 /* This macro alters the structure of the input */
 /* table to match the updated table structure.  */

  %if %sysfunc(exist(acm.&data)) %then
  %do; /* table exists - fix it */

     data acm.&data(compress=yes);
      length datetime     8
             id           8
             name      $500
             shortname $500;
      format datetime datetime.;
      set acm.&data;
     run;

  %end;/* table exists - fix it */
  %else
  %do; /* table does NOT exist */

       %tableMissing(lib=ACM,table=&data);
       
  %end;/* table does NOT exist */

%mend fixrptds;

%macro splitiomsvrs(evgid=,out=);

   %if %sysfunc(exist(ACM.IOMSERVERS)) %then
   %do; /* load acm.iomservers */
           
      %evdmres(criteria=EVG,value=&evgid);
      
      %if %sysfunc(exist(work.resources)) %then
      %do; /* resources of this type exist */

          %evdmmeas(norun=N);

          %local resourceList metricsList;

          proc sql noprint;
           select distinct id 
            into :resourceList separated by ","
             from work.resources;
           select distinct alias 
	        into :metricsList separated by " "
             from availablemetrics
              where alias ne "Availability";             
          quit;
      
          data acm.&out;
           set acm.iomservers;
           if id in (&resourceList);
           keep datetime shift id name shortname platformresource serverresource serviceresource &metricsList; 

          run;
          
          %if NOT %sysfunc(exist(acm.&out)) %then
          %do; /* confirm table was created */

             %tableMissing(lib=ACM,table=&out);

          %end;/* confirm table was created */
          
      %end;/* resources of this type exist */
      
      /* reset error flag just-in-case */
      %let evdmErr = N;   
      
   %end;/* load acm.iomservers */
   %else
   %do; /* acm.iomservers table missing */
   
      %tableMissing(lib=ACM,table=IOMSERVERS);
      
   %end;/* acm.iomservers table missing */
   
%mend splitiomsvrs;

%macro tableMissing(lib=,table=);

     %put %str(WARNING: The specified table %upcase((&lib..&table))  was NOT found; therefore, its structure could NOT be revised to the current schema.);

     %if %sysfunc(exist(WORK.SURPRISES)) %then
     %do; /* write record to exception table */

       proc sql noprint;
        insert into work.surprises 
         set libname = upcase("&lib"), 
             memname = upcase("&table"), 
             sortorder = 2,
             reason = "EXPECTED TABLE NOT FOUND";
         quit;
     %end;/* write record to exception table */
     
%mend tableMissing;

/* *******************************************************************************/
/* ************************** REPORTING PREP *************************************/
/* *******************************************************************************/

/* initialize stuff so we can generate a report */
proc datasets lib=work nolist;
 delete surprises acmsurprises;
 run;
quit;


data work.surprises;
 length libname $ 8
        memname $32
        sortorder 8
        reason  $70;
 stop;
run;


/* *******************************************************************************/
/* ************************** EVDB Library ***************************************/
/* *******************************************************************************/

/* NO ACTION NECESSARY */

/* *******************************************************************************/
/* ************************** EVDM Library ***************************************/
/* *******************************************************************************/

/* EVDM.AVAILFMT  -- NO ACTION */


/* EVDM.GROUPINFO -- RECREATE */
%include "&sasevdminstall/SASJobs/create_evgroupinfo.sas";

/* EVDM.GROUPS    -- RECREATE */
%include "&sasevdminstall/SASJobs/create_evgroups.sas";


/* EVDM.EVDMJOBS  -- MUST FOLLOW RECREATION OF EVGROUP* TABLES*/
/* EVDM.EVDMJOBS  -- ADD NEW REPORT JOBS/REMOVE OBSOLETE JOBS */
%addrptjob(action=ADD,criteria=EVG,category=%str(EV Group),id=9 ,value=9 ,out=ACM.OBJSPAWNERS,jobname=JEVG00009);
%addrptjob(action=ADD,criteria=EVG,category=%str(EV Group),id=31,value=31,out=ACM.SASAPPSERVERS ,jobname=JEVG00031);
%addrptjob(action=ADD,criteria=EVG,category=%str(EV Group),id=41,value=41,out=ACM.SASLOGICALSERVERS ,jobname=JEVG00041);
%addrptjob(action=REMOVE,jobname=JEVG00011);  /* remove ACM.IOMSERVERS from ETL */


/* EVDM.METRIC_COLLECTION_TYPE -- NO ACTION */


/* EVDM.PLATFORMTZ -- ADJUST STRUCTURE */
data evdm.platformtz;
 length origtz $200;
 set evdm.platformtz;
run;

/* EVDM.RPTSELECTIONS -- NO ACTION */
/* EVDM.SHIFTMAP      -- NO ACTION */
/* EVDM.TABLEINFO     -- NO ACTION */


/* *******************************************************************************/
/* ************************** EVDMRAW Library ************************************/
/* *******************************************************************************/

/* NO ACTION NECESSARY */

/* *******************************************************************************/
/* ************************** ACM Library ****************************************/
/* *******************************************************************************/

/* ACM.AVAILABILITY      - NO CHANGES NEEDED/TABLE RECREATED EVERY NIGHT*/
/* ACM.EVENTS            - NO CHANGES NEEDED/TABLE RECREATED EVERY NIGHT*/
/* ACM.GROUPINVENTORY    - NO CHANGES NEEDED/TABLE RECREATED EVERY NIGHT*/
/* ACM.MEASUREINVENTORY  - NO CHANGES NEEDED/TABLE RECREATED EVERY NIGHT*/
/* ACM.RESOURCEINVENTORY - NO CHANGES NEEDED/TABLE RECREATED EVERY NIGHT*/


/* ******* ACM Reporting Tables ******* */
/* Most can be handled in the same way  */

%fixrptds(data=FILEMOUNTS);
%fixrptds(data=HOSTPLATFORMS);
%fixrptds(data=HTTPCHECKS);

/* ACM.IOMSERVER   - OBSOLETE TABLE/WILL SPLIT BELOW */

%fixrptds(data=METADATASVRS);
%fixrptds(data=NETWORKINTERFACE);

/* ACM.OBJSPWANERS - NO ACTION/NEW REPORT TABLE */

/* MUST BE RUN AFTER EVDM.GROUPS/EVDMGROUPINFO HAVE BEEN UPDATED  */
/* ACM.SASAPPSERVERS     - NEW REPORT TABLE/SPLIT FROM IOMSERVERS */
%splitiomsvrs(evgid=31,out=sasappservers);

/* ACM.SASLOGICALSERVERS - NEW REPORT TABLE/SPLIT FROM IOMSERVERS */
%splitiomsvrs(evgid=41,out=saslogicalservers);


%fixrptds(data=TCSSERVERMGRS);
%fixrptds(data=WEBAPPSERVER);
%fixrptds(data=WIPDATADB);

proc sql noprint;
 create table acmsurprises as
  select "ACM" as libname, 
         memname,
         1 as sortorder,
         "UNEXPECTED TABLE FOUND.  This table may need to be updated manually." as reason
   from sashelp.vtable
     where libname eq "ACM" and 
        memname not in ("AVAILABILITY",     "EVENTS",
                        "RESOURCEINVENTORY","GROUPINVENTORY",   "MEASUREINVENTORY",
                        "FILEMOUNTS",       "HOSTPLATFORMS",    "HTTPCHECKS",       "IOMSERVERS",
                        "METADATASVRS",     "OBJSPAWNERS",      "NETWORKINTERFACE", "SASAPPSERVERS", 
                        "SASLOGICALSERVERS","TCSSERVERMGRS",    "WEBAPPSERVER",     "WIPDATADB");
quit;

proc append base=surprises data=acmsurprises force;
run;



/* *************************************************************** */
/* ************************** REPORT ***************************** */
/* *************************************************************** */
options ls=132;

proc sort data=surprises nodupkey;
 by sortorder libname memname;
run;


data _null_;
 set surprises;
 by sortorder;
 
 file log flowover;
 if _n_ eq 1 then
 do; /* write report header */

    put "****************************************************************************************************************";
    put "****************************************************************************************************************";
    put "****************************************************************************************************************";
    put "******************************         SAS Environment Manager Data Mart         *******************************";
    put "******************************     Automated Upgrade-in-Place Exception Report   *******************************";
    put "****************************************************************************************************************";
    put "****************************************************************************************************************";
    put "****************************************************************************************************************";
    put;
     
    put "The following anomalies were encountered while attempting to perform the upgrade-in-place.";
    put "These may not indicate significant problems, but they may highlight areas for further investigation.";
    put;
    put "Be sure to review the corresponding SAS LOG file for ERROR messages.  WARNING messages, especially, ";
    put "warnings about multiple/differing lengths being specified for a variable are expected and can be ignored";
    put;    
 
    put @4 "Library" @15 "Table" @48 "(Potential) Issue";
    put;
    
 end;/* write report header */
 

 put @4 libname @15 memname @48 reason;
run;


