/*****************************************************************************
 * Copyright  2015, SAS Institute Inc., Cary, NC, USA.  All Rights Reserved.
 ****************************************************************************/

// NOTE: Requires Service Architecture runtime and will not function standalone

@BaseScript(MasterETLBaseScript)

import groovy.transform.BaseScript

exitJVM = true

// Set up expected CLI syntax usage:
def cli = new CliBuilder(
    usage:  "ugipInit [-vdh]",
    header: "\nPerforms an upgrade-in-place of SAS Environment Manager " +
            "Service Architecture.\n" +
            "Options:",
    footer: "Exit status:\n" +
            "$RC_SUCCESS  if OK\n" +
            "$RC_WARNING  if minor issues are encountered\n" +
            "$RC_GENERAL_ERROR  if an error is encountered\n" +
            "$RC_INVALID_USAGE for invalid usage"
)
cli.with {
    v(longOpt:'verbose', 'Enable verbose output', required: false)
    d(longOpt:'debug', 'Enable debug output', required: false)
    h(longOpt:'help', 'Show usage information', required: false)
}

// Parse given arguments
OptionAccessor opt = cli.parse(args)
if (!opt) {
    term.error "Invalid argument(s): $args"
    exitPgm(RC_INVALID_USAGE)
}
if (opt.h) {
    cli.usage()
    exitPgm(RC_SUCCESS)
}
if (opt.v) {
    verbose = true
}
if (opt.d) {
    debug = true
    evDebug = 2
    setLogLevel LogLevel.DEBUG
}

if ( !frameworkInitialized ) {
    msg.info "The SAS Environment Manager Service Architecture is not initialized, no upgrade in place needed."
    writeProperty("ugip.status", "notNeeded", canonicalFile(emiBaseDir, 'sasev.properties'))                     
    return exitPgm(RC_SUCCESS)
}

String emiLogDir     = sysProps['emi.log.dir']
String sasHomeDir    = sysProps['sas.home']
String emiConfigDir  = sysProps['sasev.configDir']

String ugipStatus  = sysProps['ugip.status'] ?: "none"
msg.debug "ugipStatus="+ ugipStatus

msg.info "Service Architecture Upgrade-in-Place Utility - ${new Date()}"

if (ugipStatus == 'pending' || ugipStatus == 'in-progress') {

    msg.info "Pending upgrade-in-place of the Service Architecture detected..."
    
    if (ugipStatus == 'in-progress') {
        msg.debug "A previous upgrade-in-place appears to have been interrupted or failed; re-running"
    }
    
    String ugipBackupDir  = sysProps['ugip.backupdir'] ?: "none"
    if (ugipBackupDir == 'none') {
        msg.debug "Upgrade-in-place backup directory information not available"
    } else {
        term.info ""
        msg.info "The original Service Architecture deployment directory has been moved to $ugipBackupDir"
        msg.info "Existing datamart data has been moved to the current deployment"
    }
    
    writeProperty("ugip.status", "in-progress", canonicalFile(emiBaseDir, 'sasev.properties'))      
    
    // Step 3 Back-up Metadata Content
    term.info ""   
    msg.info "Preparing to back up existing SAS metadata content..."
    
    File pkgDir = canonicalFile(frameworkDir,'SASPackages');    
    File emiContentFile = canonicalFile(pkgDir, "productsFolderBackup.${_LOG_TIMESTAMP}.spk")

    if (!emiContentFile.exists()){
    
        msg.info "\tBacking up existing SAS metadata content"
        
        Map myOtherOpts=[:]
        myOtherOpts.env= [JAVA_TOOL_OPTIONS:'-Dsas.promotion.stp.includesourcecode=false']
    
        cmd = [
            "${sasHomeDir}/SASPlatformObjectFramework/9.4/ExportPackage",
            "-host", "${hq.connectionInfo.metadatahost}",
            "-port", "${hq.connectionInfo.metadataport}",
            "-user", "${hq.connectionInfo.user}", "-password", "${hq.connectionInfo.password}",
            "-log", "${emiLogDir}/ugip_emiContent_backup.${_LOG_TIMESTAMP}.log",
            "-package", "$emiContentFile",
            "-subprop",
            "-disableX11",           
            "-objects", "/Products/SAS Environment Manager"
        ]
    
        if (verbose) { msg.info "\tExporting existing SAS metadata content to [$emiContentFile]" }
    
        log.info "\tMetadata export command:"
        cmd.each { arg ->
            if (arg != "${hq.connectionInfo.password}") {
                log.info "\t  $arg"
            } else {
                log.info "\t  ********"
            }
        }
    
        int rc = exec(myOtherOpts, cmd)
        if (rc != 0) {        
            msg.error "\tError exporting package $emiContentFile.name rc = [$rc]"
            msg.error "\tCheck $logFile.name, then rerun this script"
            return exitPgm(RC_GENERAL_ERROR)
        }
    } else {
        msg.info "$emiContentFile already exists; skipping export of metadata content"
    }
    
    // Step 4 Delete metadata content   
    term.info ""
    msg.info "Deleting existing SAS Environment Manager metadata folder and contents..."
    cmd = [
        "${sasHomeDir}/SASPlatformObjectFramework/9.4/tools/sas-delete-objects",
        "-host", "${hq.connectionInfo.metadatahost}",
        "-port", "${hq.connectionInfo.metadataport}",
        "-user", "${hq.connectionInfo.user}", "-password", "${hq.connectionInfo.password}",
        "-log", "${emiLogDir}/ugip_emiContent_delete.${_LOG_TIMESTAMP}.log",
        "/Products/SAS Environment Manager",
        "-deleteContents"
    ]
    
    (verbose? msg:log).info "\tDeleting existing SAS metadata folder /Products/SAS Environment Manager and contents" 
    
    log.info "\tMetadata delete command:"
    cmd.each { arg ->
        if (arg != "${hq.connectionInfo.password}") {
            log.info "\t  $arg"
        } else {
            log.info "\t  ********"
        }
    }
    
    rc = exec(cmd)
    if (rc != 0) {
        msg.error "Error deleting SAS metadata folder /Products/SAS Environment Manager  rc = [$rc]"
        msg.error "Check $logFile.name, then rerun this script"
        return exitPgm(RC_GENERAL_ERROR)
    }
        
    
    // Step 5 Load Content SPK
    term.info ""
    msg.info "Loading Service Architecture metadata content..."

    if (canonicalFile(emiConfigDir,'SASPackages/emiContent.spk').exists() &&
        canonicalFile(emiConfigDir,'SASPackages/emiContent.subprop').exists()) {

        msg.info "\tLoading SAS Package [emiContent.spk]"

        File importLogFile = canonicalFile(emiLogDir, "ugip_emiContent_import.${_LOG_TIMESTAMP}.log")
        cmd = [
                canonicalPath(sasHomeDir, "/SASPlatformObjectFramework/9.4/ImportPackage${windows? '.exe' : ''}"),
                "-host", "${hq.connectionInfo.metadatahost}",
                "-port", "${hq.connectionInfo.metadataport}",
                "-user", "${hq.connectionInfo.user}", "-password", "${hq.connectionInfo.password}",
                "-includeACL", "-preservePaths", "-domain", "DefaultAuth", "-target", "/",
                "-disableX11",
                "-log", "$importLogFile",
                "-package", "${emiConfigDir}/SASPackages/emiContent.spk",
                "-subprop", "${emiConfigDir}/SASPackages/emiContent.subprop"
        ]

        exec cmd, { rc2, stdout, stderr ->
            if (rc2 == 0) {
                verbose? msg.info ("\tSuccess! Service Architecture metadata content and stored processes loaded, rc = [$rc2]") : ""
            } else if (rc2 == 4) {
                msg.warn "\tWarning: Service Architecture content and stored processes loaded with rc = [$rc2]"
                msg.warn "Check logs [$logFile] &"
                msg.warn "           [${importLogFile.parentFile.name}/${importLogFile.name}]"
                logCommand(cmd, rc2, stdout, stderr)
            } else {
                msg.error "Could not import Service Architecture content and stored processes, rc = [$rc2]"
                msg.error "Check logs [$logFile] &"
                msg.error "           [${importLogFile.parentFile.name}/${importLogFile.name}]"
                logCommand(cmd, rc2, stdout, stderr)
                return exitPgm(RC_GENERAL_ERROR)
            }
        }
    } else {
        msg.error "Missing Service Architecture content package or subprops file"
            return exitPgm(RC_GENERAL_ERROR)
    }

    // Step 6 Data Mart ACT
    term.info ""
    verbose? msg.info ("\tApplying Data Mart ACT to metadata folder...") : ""
    cmd = [
           canonicalPath(sasHomeDir, "/SASPlatformObjectFramework/9.4/tools/sas-set-metadata-access${windows? '.exe' : ''}"),
           "-host", "${hq.connectionInfo.metadatahost}",
           "-port", "${hq.connectionInfo.metadataport}",
           "-user", "${hq.connectionInfo.user}", "-password", "${hq.connectionInfo.password}",
           "/Products/SAS Environment Manager",
           "-addACT",
           "SAS Environment Manager Data Mart ACT"
    ]

    exec cmd, { rc2, stdout, stderr ->
        if (rc2 == 0) {
            verbose? msg.info ("\tSuccess! Data Mart ACT was applied to metadata folder, rc = [$rc2]") : ""
        } else {
            msg.error "Could not apply Data Mart ACT to folder, rc = [$rc2]"
            msg.error "Check the log file [$logFile] for details"
            logCommand(cmd, rc2, stdout, stderr)
            return exitPgm(RC_GENERAL_ERROR)
        }
    }

    // Step 1 Install Kits
    term.info ""
    msg.info "Updating kits..."
    String[] parms = [ '-fqx', '-k', 'ALL', '--filesOnly' ]
    kitsRC= runGroovy(canonicalPath(frameworkDir, "Scripts/evKitInstaller.groovy"), parms)
        
    if (kitsRC != RC_SUCCESS) {
         if (kitsRC== RC_WARNING) {
             log.warn "Update of kits completed with warnings. Check $logFile for details"
        } else {
            msg.error "Error rc = [${kitsRC}] while updating kits [${parms}], check log."
            return exitPgm(RC_GENERAL_ERROR)
        }
    } else {
        log.debug "\tupdate of kits completed successfully"
    }

    // Step 2 Update Data Schema     
    term.info ""
    msg.info "Starting Data Mart Schema Migrations..."

    commonSASOptions.logDir     = logDir.absolutePath
    commonSASOptions.verbose    = verbose
    commonSASOptions.args       = COMMON_SAS_ARGS

    long start = System.currentTimeMillis()

    Map myOpts = [:] << commonSASOptions
    myOpts['autoexec'] = canonicalPath(datamartDir, "EVDMAUTOEXEC_BATCH.sas")

    if (acmInitialized) {
        msg.info "\tACM is initialized; updating data schema."    
                File sasProg = canonicalFile(jobsDir, "ugip_acm_datamart.sas")
            
                verbose? msg.info("Executing ugip_acm_datamart.sas") : ""
                int rc = sas.run myOpts, sasProg
                if (rc > 1) {
                    term.error "ugip_acm_datamart.sas execution failed with return code $rc"
                    return exitPgm(RC_GENERAL_ERROR)
                }

    } else {
        msg.info "\tACM is NOT initialized; no schema update needed."
    }

    if (apmInitialized) {
        msg.info "\tAPM is initialized; updating data schema."    
                File sasProg = canonicalFile(jobsDir, "ugip_apm_datamart.sas")
            
                verbose? msg.info("Executing ugip_apm_datamart.sas") : ""
                int rc = sas.run myOpts, sasProg
                if (rc > 1) {
                    term.error "ugip_apm_datamart.sas execution failed with return code $rc"
                    return exitPgm(RC_GENERAL_ERROR)
                }
    } else {
        msg.info "\tAPM is NOT initialized; no schema update needed."
    }

    

    // Step 7 Update APM ETL resource to point to the correct metrics directory
    def apm = hq.getSingletonResource('SAS Environment Manager Data Mart 9.4 APM ETL Processing')
    if (apm) {
        (verbose? msg:log).info "Updating the APM ETL Processing Resource..."
        String apmMetricsDir = canonicalPath(frameworkDir, 'Logs/metrics')
        Map apmConfigMap = [
            'metrics.dir':apmMetricsDir,
            'program':"master_apm_etl$scriptExtension",
            'emi.type':'framework'
        ]
        if (hq.updateConfig(apm, apmConfigMap)) {
            log.info "Successfully updated APM ETL metrics.dir config to $apmMetricsDir"
        } else {
            msg.error "Unable to update APM ETL Processing configuration. Check $logFile.name"
            return exitPgm(RC_GENERAL_ERROR)
        }
    } else {
        msg.warn "No APM ETL Processing resource found"
    }
    
    term.info ""
    msg.info "The upgrade-in-place of the Service Architecture completed successfully"
    writeProperty("ugip.status", "completed", canonicalFile(emiBaseDir, 'sasev.properties'))                     
    return exitPgm(RC_SUCCESS)
} else {
    term.info ""
    msg.info "An upgrade-in-place of the Service Architecture does NOT appear to be pending."
    return exitPgm(RC_SUCCESS)
}



def exitPgm(int rc) {
    term.info ""
    msg.info "Exiting Service Architecture Upgrade-in-Place Utility"
    
    if (rc == RC_GENERAL_ERROR){     
        writeProperty("ugip.status", "pending", canonicalFile(emiBaseDir, 'sasev.properties'))      
        msg.info "The upgrade-in-place of the Service Architecture did NOT complete successfully"
        msg.warn "Check $logFile.name for details, correct any issues and rerun this script"
    }

    if (exitJVM) {
        System.exit rc
    } else {
        return rc
    }
}

def logCommand(def cmd, int rc, String stdout, String stderr) {
    log.info "Command details:"
    boolean isPassword = false
    // Try to avoid logging password values
    cmd.each { arg ->
        if (isPassword) {
            log.info "\t********"
            isPassword = false
        } else {
            log.info "\t$arg"
        }
        if (arg =~ /(?i)password/) {
            isPassword = true
        }
    }
    log.info "Return code: $rc"

    log.info "Standard Output:"
    stdout.eachLine {
        log.info "\t$it"
    }
    log.info "Standard Error:"
    stderr.eachLine {
        log.info "\t$it"
    }
}
